/*
 * Classified Tydeman Consulting Proprietary
 * Copyright (C) 1998, 2008 Tydeman Consulting.  All rights reserved.
 *
 * Fred J. Tydeman	tydeman@tybor.com
 *
 * This is a dummy <fenv.h>.  It would be used if the compiler you are
 * testing is missing <fenv.h>.  It allows most of the FPCE Test Suite
 * tests to compile and run.  To use this file, in your *.bug file, do:
 *   #define FENV_H "fenvdumy.h"
 *
 * Another way is to do -DFENV_MISSING (or equivalent) on the compiler's
 * command line to define FENV_MISSING.
 *
 * Note:  FIX_FENV_FUNCTIONS is NOT defined as there are (dummy)
 * fenv.h functions defined here.
 *
 * MODIFICATION HISTORY:
 *
 * 1998/07/07 - Create from fenvsamp.h
 * 1998/07/22 - Redo rounding related items.
 * 1998/07/22 - feholdexcept must return int.
 * 1998/12/19 - feholdexcept now returns 0 (was 1) for success.
 * 1998/12/19 - fesetround now returns 0 (was 1) for success.
 * 2000/03/01 - Make classification explicit in each file.
 * 2000/03/01 - Remove confidential so can post to public ftp site.
 * 2002/01/07 - Update to align with TC1 (DR 202) of C99.
 * 2004/05/05 - Update return value from feclearexcept (was (e)).
 * 2008/07/14 - Shorten marco symbols to less than 32 chars.
 *
 ***********************************************************************/

#ifndef _FENV_H_
#define _FENV_H_ 1

typedef int fenv_t;
typedef int fexcept_t;
	
#define FE_INEXACT	0x0020
#define FE_UNDERFLOW	0x0010
#define FE_OVERFLOW	0x0008
#define FE_DIVBYZERO	0x0004
#define FE_INVALID	0x0001
	
#define FE_ALL_EXCEPT	0x003d

#define FE_TONEAREST	0x0000
#define FE_DOWNWARD	0x0400
#define FE_UPWARD	0x0800
#define FE_TOWARDZERO	0x0c00

#define FE_DFL_ENV (fenv_t *)0
#define FIX_USABLE_FE_DFL_ENV	/* nothing to test */

/****************************************************************/
 
#define feclearexcept(e) (0)		/* as if all OK */
#define FIX_TRUE_feclearexcept	/* only macro */
#define FIX_FLAG_feclearexcept	/* Too few or many flags cleared */

#define fetestexcept(e) 0		/* as if nothing raised */
#define FIX_TRUE_fetestexcept	/* only macro */
#define FIX_FLAG_fetestexcept	/* Too few or many flags returned */

#define feraiseexcept(e) (e)		/* as if nothing raised */
#define FIX_TRUE_feraiseexcept	/* only macro */
#define FIX_FLAG_feraiseexcept	/* Too few or many flags raised */
#define FIX_FLAGS_feraiseexcept	/* No flags raised by feraise*() */

#define FIX_FE_FLAGS	/* flags are untestable */

/****************************************************************/

#define fegetexceptflag(p,i) (*p = i,1)
#define FIX_TRUE_fegetexceptflag	/* only macro */

#define fesetexceptflag(p,i) 1
#define FIX_TRUE_fesetexceptflag	/* only macro */

#define FIX_FE_EXCEPT	/* fe*exceptflag wrong */

/****************************************************************/

#define fegetround() FE_TONEAREST
#define FIX_TRUE_fegetround	/* only macro */
/* #define FIX_RND_fegetround	// it is correct rnding dir */

#if (19980626uL <= FPCE_TEST_LVL)
 #define fesetround(i) (FE_TONEAREST != (i))
#else
 #define fesetround(i) (FE_TONEAREST == (i))
#endif
#define FIX_TRUE_fesetround	/* only macro */
#define FIX_RND_fesetround		/* rnding dir not changed */

/****************************************************************/

#define fegetenv(p) (*p = 0,1)
#define FIX_TRUE_fegetenv	/* only macro */

#define fesetenv(p) 1
#define FIX_TRUE_fesetenv	/* only macro */

#define feupdateenv(p) 1
#define FIX_TRUE_feupdateenv	/* only macro */

#if (19980626uL <= FPCE_TEST_LVL)
 #define feholdexcept(p) 0
#else
 #define feholdexcept(p) 1
#endif
#define FIX_TRUE_feholdexcept	/* only macro */

#define FIX_FE_ENV		/* fe*env wrong */

/****************************************************************/

#endif /* _FENV_H_ */
