/*
 *  Classified Tydeman Consulting Proprietary
 *  Copyright (C) 1996, 2014 Tydeman Consulting.  All rights reserved.
 *
 *  Fred J. Tydeman	tydeman@tybor.com
 *
 * - -- --- ---- ----- -------------------------------------------------
 *
 *    This code is copyrighted.  Under the copyright laws, this code may
 *         not be copied, in whole or part, without prior written
 *         consent of Tydeman Consulting.
 *
 *    RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the
 *         government is subject to restrictions as set forth in
 *         subparagraph (c)(1)(ii) of the Rights in Technical Data and
 *         Computer Software clause at DFARS 252.227-7013 and
 *         FAR 52.227-19.
 *
 * - -- --- ---- ----- -------------------------------------------------
 * $Revision: 1.1 $
 ***********************************************************************
 *  FILE: v1150000.h	which is included by g1150000.h
 *
 *  Test vectors for sin == sine == for any type, any precision.
 *
 *  MODIFICATION HISTORY:
 *
 * 1996/01/15 - Create. First 5 spots reserved for Signaling NaNs
 * 1996/03/14 - Better symbols for various ULP meanings.
 * 1996/05/12 - Add tests for MAX and MAXm1.
 * 1996/06/02 - Add missing 'L' suffix on some arguments.
 * 1996/06/07 - Removed some PREC_q tests; exceeded ANSI limit.
 * 1996/06/14 - Added tests close to zero.
 * 1996/06/17 - Split into IEEE and general.
 * 1997/01/14 - Add FPK() macro usage.
 * 1997/01/26 - *_t -> *_q (reserved by POSIX.1).
 * 1997/09/17 - Update email address.
 * 1997/10/03 - Add IMPL_INEXACT_*.
 * 1997/10/04 - Add DEN_FLAGS_OPT.
 * 1999/05/17 - Had MAXm1 and MAX swapped for 64 and 113 bit precisions.
 * 1999/05/17 - Add MAXm1 and MAX as decimal for additional testing.
 * 2000/03/01 - Make classification explicit in each file.
 * 2000/03/01 - Remove confidential so can post to public ftp site.
 * 2002/01/18 - Exclude more too large numbers.
 * 2004/07/18 - Add FIX_CONST_EXP_IN_INIT.
 * 2006/04/10 - Move MIN_q test to here from v1150001 (may underflow).
 * 2006/04/18 - Add FP_EXP(), in particular, for uses of DEN_*
 * 2006/04/18 - Update DEBUG section.
 * 2006/04/18 - Indent #if's.
 * 2006/06/01 - Add guard around # define TESTING_IEEE_VECTORS.
 * 2007/06/18 - Remove FIX_FLUSH_DENORM for FP_EXP(x).
 * 2008/02/12 - Add DFP items.
 * 2008/02/12 - Indent #if's.
 * 2008/06/13 - Add another FIX_BIGGER_THAN_FPMAX.
 * 2008/07/14 - Shorten marco symbols to less than 32 chars.
 * 2008/08/11 - Add MIN_FLAGS, MIN_FLAGS_OPT.
 * 2010/03/13 - Add 107 for PREC_q.
 * 2010/06/30 - INF_q -> INF_Q.
 * 2010/06/30 - NAN_q -> NAN_Q.
 * 2014/06/17 - IMPL_SUPER_NORMAL in place of 107 PREC_q.
 * 2014/07/24 - FLUSH_DENORM -> FLUSH_RUN_DEN_BOTH.
 * 2014/07/24 - FP_EXP(MIN_q+DEN_q) -> MINp1_q
 * 2014/07/24 - Move FP_EXP(x) to g1*.h.
 * 2014/08/01 - Put constant before ==.
 * 2014/08/01 - Allow for many flavors of DBL_DBL.
 * 2014/08/02 - Allow for SLOPPY or WRONG MAX_LDBL.
 * 2014/08/04 - MIN_q-DEN_q -> MINm1_Q.
 * 2014/08/04 - DEN_q -> DEN_Q.
 * 2014/08/04 - MINp1_q -> MINp1_Q.
 * 2014/08/04 - MIN_q -> MIN_Q
 * 2014/08/08 - Indent #if's.
 * 2014/08/08 - Add FILLER.
 * 2014/08/18 - ULP_q -> EPS_q  and  ULP_f -> ERR_f
 * 2014/09/13 - ULP_* -> ERR_*.
 * 2014/12/28 - FIX_SLOPPY_MAX_LDBL -> FIX_SLOPPY_SUP_LDBL.
 *
 ***********************************************************************/

#ifdef DEBUG	/* local override of FIXes to force errors to show */
  #ifdef DEBUG2	/* Kills some compilers */
    #undef  FIX_BIGGER_THAN_FPMAX
  #endif
#endif
#ifdef DEBUG	/* local override of FIXes to force errors to show */
  #ifdef DEBUG3
/*  #undef  FIX_FLUSH_RUN_DEN_BOTH */
/*  #undef  IMPL_MAX2_LDBL */
/*  #undef  IMPL_MAX3_LDBL */
  #endif
  #undef  FIX_SLOPPY_SUP_LDBL
  #undef  FIX_WRONG_SUP_LDBL
#endif

#ifndef   TESTING_IEEE_VECTORS
  #define TESTING_IEEE_VECTORS
#endif

#define FILLER        ZERO_q,               ZERO_q,  BIT4BIT,          0,         0

/*  0*/{               NAN_Q,                NAN_Q,  ERR_NAN, SNAN_FLAGS, NAN_FLAGS },
/*  1*/{              -NAN_Q,               -NAN_Q,  ERR_NAN, SNAN_FLAGS, NAN_FLAGS },
/*  2*/{          /*2*/NAN_Q,           /*2*/NAN_Q,  ERR_NAN, SNAN_FLAGS, NAN_FLAGS },
/*  3*/{          /*3*/NAN_Q,           /*3*/NAN_Q,  ERR_NAN, SNAN_FLAGS, NAN_FLAGS },
/*  4*/{          /*4*/NAN_Q,           /*4*/NAN_Q,  ERR_NAN, SNAN_FLAGS, NAN_FLAGS },
/*  5*/{               NAN_Q,                NAN_Q,  ERR_NAN,          0, NAN_FLAGS },
/*  6*/{              -NAN_Q,               -NAN_Q,  ERR_NAN,          0, NAN_FLAGS },
/*  7*/{          /*7*/NAN_Q,           /*7*/NAN_Q,  ERR_NAN,          0, NAN_FLAGS },
/*  8*/{          /*8*/NAN_Q,           /*8*/NAN_Q,  ERR_NAN,          0, NAN_FLAGS },
/*  9*/{          /*9*/NAN_Q,           /*9*/NAN_Q,  ERR_NAN,          0, NAN_FLAGS },

/* 10*/{        /*10*/+INF_Q,                NAN_Q, SOME_NAN, FE_INVALID,         0 },
/* 11*/{        /*11*/-INF_Q,                NAN_Q, SOME_NAN, FE_INVALID,         0 },

/* 12*/{             -ZERO_q,              -ZERO_q,   ERR_M0,          0,         0 },

#if (defined FIX_FLUSH_RUN_DEN_BOTH)
/* 13*/{ FILLER },
/* 14*/{ FILLER },
#else
/* 13*/{         /*13*/DEN_Q,          /*13*/DEN_Q,  ERR_DEN, IMPL_INEXACT_sin|DEN_FLAGS, FE_INEXACT|DEN_FLAGS_OPT },
/* 14*/{             MINm1_Q,              MINm1_Q,  ERR_DEN, IMPL_INEXACT_sin|DEN_FLAGS, FE_INEXACT|DEN_FLAGS_OPT },
#endif
/* 15*/{               MIN_Q,                MIN_Q,    ERR_f, IMPL_INEXACT_sin|MIN_FLAGS, FE_INEXACT|MIN_FLAGS_OPT },
/* 16*/{             MINp1_Q,              MINp1_Q,    ERR_f, IMPL_INEXACT_sin, FE_INEXACT },

/*
 * Check right at overflow point +/- 1 ulp in argument.
 */
#if 2 == FP_RADIX

  #if 24 == PREC_q
    #define SINMAXm1 FPK(-0.6421992576228771706639908883438606607000)  /* BF24672C */
    #define SINMAX   FPK(-0.5218765233336585405515053570198067229357)  /* BF0599B3 */

  #elif 53 == PREC_q
    #define SINMAXm1 FPK(+0.9270279254337350379083526842005971419045)  /* 3FEDAA3677C6EE8A */
    #define SINMAX   FPK(+.004961954789184061790502671197074705750764)  /* 3F7452FC98B34E97 */

  #elif 64 == PREC_q
    #define SINMAXm1 FPK(+0.5219486624350058793178905118729959683573)  /* 3FFE859E6D73596B5481 */
    #define SINMAX   FPK(+0.9921510991235264959426152209755116090882)  /* 3FFEFDFD9D4B6D0E5F7C */

  #elif (106 == PREC_q)
    #define SINMAXm1 FPK(-0.6637068955153486851521855004242787077671)
    #define SINMAX   FPK(-0.8197920330577909469821687189650515296750)

  #elif (107 == PREC_q)
    #define SINMAXm1 FPK(-0.8197920330577909469821687189650515296750)
    #define SINMAX   FPK(-0.8813395094470001212842578621525391206053)

  #elif 113 == PREC_q
    #define SINMAXm1 FPK(+0.1762155772170363718124666989569090523778)  /* 3FFC68E3B6698B20EF2F7C7282BDCD04 */
    #define SINMAX   FPK(+0.9519148540788204811363248929375729420329)  /* 3FFEE761623DB0B6FFC87A2204A2B017 */

  #else
    #error Need to add precision PREC_q to v1150000.h
  #endif

#elif 10 == FP_RADIX

  #if 7 == PREC_q
    #define SINMAXm1 FPK(+0.8850718107957575838301674028534298067868)
    #define SINMAX   FPK(+0.5532570915147650073405992289589488483283)

  #elif 16 == PREC_q
    #define SINMAXm1 FPK(+0.7490020594422313032918779743779447074144)
    #define SINMAX   FPK(+0.1094503281143336095673279797935442227075)

  #elif 34 == PREC_q
    #define SINMAXm1 FPK(-2.166641878627115047092618278862443362984E-1)
    #define SINMAX   FPK(+0.5582907749092521238056875912416942232626)

  #else
    #error Contact Tydeman
  #endif
#else
  #error Contact Tydeman
#endif
/* 17*/{             MAXm1_q,             SINMAXm1,    ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 18*/{               MAX_q,               SINMAX,    ERR_f, IMPL_INEXACT_sin, FE_INEXACT },

#if (106 == PREC_q) || (107 == PREC_q)
  #if   (defined FIX_SLOPPY_SUP_LDBL) || (defined FIX_WRONG_SUP_LDBL)	/* skip */
  #elif (defined IMPL_MAX3_LDBL)	/* SUP_LDBL == 3*MAX_DBL */
    #define SINSUP   FPK(0.01488537569448941853668033599711902162278)
  #elif (defined IMPL_MAX2_LDBL)	/* SUP_LDBL == 2*MAX_DBL */
    #define SINSUP   FPK(-9.923787409350446540191668498621061341700E-3)
  #endif
#endif
#if (!defined SINSUP)
/* 19*/{ FILLER },
#else
/* 19*/{               SUP_q,               SINSUP,    ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
#endif

#if 2 == FP_RADIX
/* 20*/{ FPK(3.40282326356119256160033759537265639424E38),
	   FPK(-.6421992576228771706639908883438606607000),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 21*/{ FPK(3.40282346638528859811704183484516925440E38),
	   FPK(-.5218765233336585405515053570198067229357),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },

  #if !((defined FIX_BIGGER_THAN_FPMAX))
    #if 53 <= PREC_q
/* 22*/{ FPK(1.797693134862315508561243283845062402343E308),
	   FPK(0.9270279254337350379083526842005971419045),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 23*/{ FPK(1.797693134862315708145274237317043567980E308),
	   FPK(0.004961954789184061790502671197074705750764),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },

      #if (64 <= PREC_q) && (106 != PREC_q) && (107 != PREC_q)
/* 24*/{ FPK(1.189731495357231764956768379433933279574E4932),
	   FPK(0.5219486624350058793178905118729959683573),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 25*/{ FPK(1.189731495357231765021263853030970205169E4932),
	   FPK(0.9921510991235264959426152209755116090882),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
      #else
/* 24*/{ FILLER },
/* 25*/{ FILLER },
      #endif

      #if 106 <= PREC_q
/* 26*/{ FPK(1.797693134862315907729305190788980417060E308),
	   FPK(-.6637068955153486851521855004242787077671),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 27*/{ FPK(1.797693134862315907729305190789002575339E308),
	   FPK(-.8197920330577909469821687189650515296750),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },

        #if 113 <= PREC_q
/* 28*/{ FPK(1.189731495357231765085759326628006901629E4932),
	   FPK(0.1762155772170363718124666989569090523778),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 29*/{ FPK(1.189731495357231765085759326628007016196E4932),
	   FPK(0.9519148540788204811363248929375729420329),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
        #else
/* 28*/{ FILLER },
/* 29*/{ FILLER },
        #endif
      #else
/* 26*/{ FILLER },
/* 27*/{ FILLER },
/* 28*/{ FILLER },
/* 29*/{ FILLER },
      #endif
    #else
/* 22*/{ FILLER },
/* 23*/{ FILLER },
/* 24*/{ FILLER },
/* 25*/{ FILLER },
/* 26*/{ FILLER },
/* 27*/{ FILLER },
/* 28*/{ FILLER },
/* 29*/{ FILLER },
    #endif
  #else
/* 22*/{ FILLER },
/* 23*/{ FILLER },
/* 24*/{ FILLER },
/* 25*/{ FILLER },
/* 26*/{ FILLER },
/* 27*/{ FILLER },
/* 28*/{ FILLER },
/* 29*/{ FILLER },
  #endif

#elif 10 == FP_RADIX

  #if !((defined FIX_BIGGER_THAN_FPMAX))
/* 20*/{ FPK(9.999998E96),
	   FPK(+0.8850718107957575838301674028534298067868),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 21*/{ FPK(9.999999E96),
	   FPK(+0.5532570915147650073405992289589488483283),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },

    #if 16 <= PREC_q
/* 22*/{ FPK(9.999999999999998E384),
	   FPK(+0.7490020594422313032918779743779447074144),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 23*/{ FPK(9.999999999999999E384),
	   FPK(+0.1094503281143336095673279797935442227075),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },

      #if 34 <= PREC_q
/* 24*/{ FPK(9.999999999999999999999999999999998E6144),
	   FPK(-2.166641878627115047092618278862443362984E-1),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 25*/{ FPK(9.999999999999999999999999999999999E6144),
	   FPK(+0.5582907749092521238056875912416942232626),
	   ERR_f, IMPL_INEXACT_sin, FE_INEXACT },
/* 26*/{ FILLER },
/* 27*/{ FILLER },
/* 28*/{ FILLER },
/* 29*/{ FILLER },
      #else
/* 24*/{ FILLER },
/* 25*/{ FILLER },
/* 26*/{ FILLER },
/* 27*/{ FILLER },
/* 28*/{ FILLER },
/* 29*/{ FILLER },
      #endif
    #else
/* 22*/{ FILLER },
/* 23*/{ FILLER },
/* 24*/{ FILLER },
/* 25*/{ FILLER },
/* 26*/{ FILLER },
/* 27*/{ FILLER },
/* 28*/{ FILLER },
/* 29*/{ FILLER },
    #endif
  #else
/* 20*/{ FILLER },
/* 21*/{ FILLER },
/* 22*/{ FILLER },
/* 23*/{ FILLER },
/* 24*/{ FILLER },
/* 25*/{ FILLER },
/* 26*/{ FILLER },
/* 27*/{ FILLER },
/* 28*/{ FILLER },
/* 29*/{ FILLER },
  #endif
#else
  #error Contact Tydeman
#endif
